unit Base;

{******************************************************************************
 Unit Name:	 Base
 Description:Base contains an example of use of TDeriver. TBase class and
 						 its derived classes will provide polymorphic creation.
 Author:     M. Venturini
 Revisor:
 Date:       17 Feb. 97
 Revision:
******************************************************************************}

interface

uses
	Deriver;

type

	{TBaseCreateInfo contains creational infos. If some derived class needs more
   infos TBaseCreateInfo can be extended with no modification to this unit.   }
	TBaseCreateInfo = class(TObject)
  public
  	Key: Integer; {Example of info necessary to select which class istantiate}
    {...}
  end;

  {A generic base class...}
  TBase = class;
  TBaseClass = class of TBase;
	TBase = class(TObject)
	private
    {...}
	protected
    {...}
		class function Match(const Info: TObject): Boolean; virtual;
  public
    {...}
		class	procedure Register;
		class function DerivedClass(const Info: TBaseCreateInfo): TBaseClass;
    constructor Create(const Info: TBaseCreateInfo); virtual;
    destructor Destroy; override;
  end;

implementation

{An instance of TDeriver must be included at the unit level}
var
	ClassDeriver: TDeriver;

{The class method Match is required to examin creational infos. If they match
 the requirements for this class true is returned.                            }
class function TBase.Match(const Info: TObject): Boolean;
begin

	with Info as TBaseCreateInfo do
		Result := (Key = 0);

end;

{The class method Register must be defined to permit the derived classes to
 register themself without any direct reference to the unit level instance of
 TDeriver and to pass automatically the correct registration parameters.      }
class	procedure TBase.Register;
begin

	ClassDeriver.Register(Self,Self.Match);

end;

{The class method DerivedClass must be defined to cast the generic
 DerivedClass method of TDerived to this class class reference.               }
class function TBase.DerivedClass(const Info: TBaseCreateInfo): TBaseClass;
begin

	Result := TBaseClass(ClassDeriver.DerivedClass(Info));

end;

{...}

{The class contructor must have a TBaseCreateInfo (derived class may need it)
 and call the IncInstance method of the unit level instance of TDeriver.       }
constructor TBase.Create(const Info: TBaseCreateInfo);
begin

 	inherited Create;
	ClassDeriver.IncInstance; {Note: in Delphi 2 may be replaced with a finalization}
  {...}

end;

{The class destructor must call the DecInstance method of the unit level
 instance of TDeriver.                                                         }
destructor TBase.Destroy;
begin

  {...}
	ClassDeriver.DecInstance;  {Note: in Delphi 2 may be replaced with a finalization}
	inherited Destroy;

end;

initialization

	{Initialization of the unit level instance of TDeriver and TBase registration}
	ClassDeriver := TDeriver.Create;
	TBase.Register;

{Note: in Delphi 2 a finalization can be used to free the unit level instance of TDeriver}

end.

